# Ce script produit un csv des données pertinentes de l'OCI.
# Les différences entre ce csv et les objets produits par les scripts suivants (produisant l'analyse morphosyntaxique):
  # 1- Les données du csv excluent les informations protégées par la propriété intellectuelle.
  # 2- Elles couvrent l'ensemble des 6 premiers mois de chaque année (2017 à 2020), 
        # Les données pour l'analyse morphosyntaxique couvre seulement de mars à juin (4 mois par an au lieu de 6)


package_list <- c("mongolite","data.table","tidyr")
for(p in package_list){
  if (p %in% installed.packages()==FALSE){install.packages(p,dependencies = TRUE)}
  library(p,character.only=TRUE)
}

accesssible_data_folder <- "data/"

# Connexions à la base de données
DB_connection_2020 <- mongolite::mongo(db = "projetThinkTank", collection = "all_articles_01122019_30062020")
DB_connection_2019 <- mongolite::mongo(db = "projetThinkTank", collection = "all_articles_01122018_30062019")
DB_connection_2018 <- mongolite::mongo(db = "projetThinkTank", collection = "all_articles_01012018_30062018")
DB_connection_2017 <- mongolite::mongo(db = "projetThinkTank", collection = "all_articles_01012017_30062017")


fields = '{"scraped_on" : true, "media_url" : true, "full_article_text" : true,
           "scraped_on" : true, "original_article_url" : true, "title" : true}'

dt_articles_2020 <- DB_connection_2020$find(query = "{}", fields = fields) %>% data.table()
dt_articles_2020[, date := as.Date(as.POSIXct(scraped_on, origin="1970-01-01"))]
DB_connection_2020$disconnect()

dt_articles_2019 <- DB_connection_2019$find(query = "{}", fields = fields) %>% data.table()
dt_articles_2019[, date := as.Date(as.POSIXct(scraped_on, origin="1970-01-01"))]
DB_connection_2019$disconnect()


dt_articles_2018 <- DB_connection_2018$find(query = "{}", fields = fields) %>% data.table()
dt_articles_2018[, date := as.Date(as.POSIXct(scraped_on, origin="1970-01-01"))]
DB_connection_2018$disconnect()

#dt_articles_2018 <- dt_articles_2018[date >= as.Date("2018-03-01")]

dt_articles_2017 <- DB_connection_2017$find(query = "{}", fields = fields) %>% data.table()
dt_articles_2017[, date := as.Date(as.POSIXct(scraped_on, origin="1970-01-01"))]
DB_connection_2017$disconnect()


dt_articles_2017_2019 <- rbind(dt_articles_2017, dt_articles_2018, dt_articles_2019)
rm(dt_articles_2017); rm(dt_articles_2018); rm(dt_articles_2019)
gc()

liste_media <- read.csv(paste0(accesssible_data_folder,"liste_finale_medias.csv")) %>% data.table()
liste_media <- liste_media[dans_oci == TRUE]

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#### Nettoyage pour usage général ####
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dt_articles_2017_2019$corpus <- 1
dt_articles_2020$corpus <- 2
dt_full_text <- rbind(dt_articles_2017_2019, dt_articles_2020)
rm(dt_articles_2017_2019); rm(dt_articles_2020); gc()
setnames(dt_full_text, "_id", "doc_ID")
#D'abord on met la date dans le bon format

dt_full_text[, date := as.Date(as.POSIXct(scraped_on, origin="1970-01-01"))][,scraped_on := NULL]

#On garde seulement les articles de plus de 200 caractères
dt_full_text[,text_length := str_length(full_article_text)]
dt_full_text <- dt_full_text[text_length > 200][,text_length := NULL]


#On prend seulement les médias que l'on veut dans notre corpus
dt_full_text <- take_medias_in_list(dt_full_text, liste_media)
#Certaines sections de média (sections affaires, argent, etc.) ne sont pas retirés par notre fonction
other_media_to_take_off <- c("https://www.985fm.ca/sports",
                             "https://www.tvanouvelles.ca/argent",
                             "http://www.lapresse.ca/techno/",
                             "https://www.lapresse.ca/affaires/",
                             "http://affaires.lapresse.ca/dossiers/la-presse-affaires-magazine/",
                             "http://affaires.lapresse.ca/",
                             "https://www.journalsaint-francois.ca/the-gleaner-changed-hands-wednesday/",
                             "http://www.estrieplus.com/section-0404040431353537-actualites.html")

dt_full_text <- dt_full_text[!media_url %in% other_media_to_take_off][,media_url := NULL]


# Les acronymes qui reviennent souvent dans le corpus (CHSLD, SRAS, OMS, PCU)
# On veut éviter que des parties de la forme longue se retrouve dans les bigrams.
dt_full_text[,full_article_text := str_replace_all(full_article_text, 
                                                   pattern = "[Cc]entre[s]?(\\s)(d['’]hébergement|hospitaliers)(\\s)(et )?(de|en)(\\s)soins(\\s)(de(\\s))?longue(\\s)durée", 
                                                   replacement = "CHSLD")]
dt_full_text[,full_article_text := str_replace_all(full_article_text, 
                                                   pattern = "[Pp]restation(\\s)canadienne(\\s)d[’']urgence", 
                                                   replacement = "PCU")]
dt_full_text[,full_article_text := str_replace_all(full_article_text, 
                                                   pattern = "[Ss]yndrome(\\s)respiratoire(\\s)aigu(\\s)sévère", 
                                                   replacement = "SRAS")]
dt_full_text[,full_article_text := str_replace_all(full_article_text, 
                                                   pattern = "[Oo]rganisation(\\s)[Mm]ondiale(\\s)de(\\s)la(\\s)[Ss]anté", 
                                                   replacement = "OMS")]

# Detection des articles sur la COVID
dt_full_text[(grepl("covid", full_article_text, ignore.case = TRUE) | 
                grepl("virus", full_article_text, ignore.case = TRUE) |
                grepl("sars-cov", full_article_text, ignore.case = TRUE) |
                grepl("sras-cov", full_article_text, ignore.case = TRUE))
             & corpus == 2, #Ils ne peuvent être que dans le second corpus (2020)
             is_covid := TRUE][is.na(is_covid), is_covid := FALSE]

# Detection des articles sur la science
dt_full_text[grepl("(\\s)scien", full_article_text, ignore.case = TRUE) |
               grepl("(\\s)chercheu", full_article_text, ignore.case = TRUE),
             is_science := TRUE][is.na(is_science), is_science := FALSE]

dt_full_text <- dt_full_text[,list(doc_ID, date, root_url, is_covid, is_science, n_char_text = nchar(full_article_text))]

fwrite(dt_full_text, file = paste0(accesssible_data_folder, "dt_full_text_for_count_sci.csv"))
#save(dt_full_text, file = paste0(accesssible_data_folder, "dt_full_text_for_count_sci.RData"))