
#Chargement des librairies
library(dplyr)
library(data.table)
library(tidyr)
library(stringr)
library(tm)
library(tidytext)
library(doParallel)
library(splitstackshape)
library(ggplot2)
library(udpipe)

#### Global variables ####
data_folder <- "/data/covid/"

path_to_UDPIPE_model <- "a_completer"

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#### Préparation des données ####
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load(paste0(data_folder, "dt_full_text.RData"))

#On enlève les '*' dans le full_article_text
dt_full_text[,full_article_text := str_replace_all(full_article_text, "\\*", "")]
#On veut dans un premier temps n'utiliser que les articles qui parlent de science
dt_science <- dt_full_text[is_science == TRUE, list(art_ID, full_article_text, corpus)] %>% unique()
#On divise par paragraphe
dt_science <- cSplit(dt_science[,c("art_ID", "corpus", "full_article_text")], 
                     "full_article_text", "\n\n", direction = "long") %>% data.table()

setnames(dt_science, "full_article_text", "paragraphe")
#On ne prend que les paragraphes qui mentionnent scien ou chercheu
dt_science <- dt_science[grepl("(\\s)scien", paragraphe) | grepl("(\\s)chercheu", paragraphe)]
#On fait des identifiants de paragraphe pour la recherche sémantique ensuite
dt_science[,paragraphe_ID := paste0(art_ID, c(1:.N)), by = art_ID]

save(dt_science, file = paste0(data_folder, "dt_science.RData"))


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#### Analyse morphosyntaxique du corpus science ####
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#Importation du modèle de langue 
#le modèle a été préalablement téléchargé grâce à la commande :
#udpipe_download_model(language = "french-gsd", model_dir = model_dir))
udmodel <- udpipe_load_model(file = path_to_UDPIPE_model)

#On crée la table avec de tokens annotés
dt_tagged_tokens_science <-
  #Appliquer le partitionnement et l'analyse morphosyntaxique
  udpipe_annotate(
    udmodel,
    dt_science$paragraphe,
    doc_id = dt_science$paragraphe_ID,
    parallel.cores = 8) %>%
  as.data.table(.) %>%
  # on retire les tokens sans annotation
  .[!is.na(upos)]

dt_tagged_tokens_science <- merge(dt_tagged_tokens_science[,list(doc_id, token_id, token, lemma, upos, feats, dep_rel)],
                                  dt_science[,list(art_ID, paragraphe_ID, corpus)],
                                  by.x = "doc_id", by.y = "paragraphe_ID",
                                  all.x = TRUE, all.y = FALSE)

save(dt_tagged_tokens_science, file = paste0(data_folder, "dt_tagged_tokens_science.RData"))
