# Script pour produire différents fichiers des données prétraitées de Facebook

library(data.table)
library(tidyr)
library(stringr)
library(udpipe)

data_folder <- "/data/covid/"
accesssible_data_folder <- "data/"



fb_2017 <- read.csv(paste0(data_folder, "postsFBcovid-2017.csv")) %>% data.table()
fb_2018 <- read.csv(paste0(data_folder,"postsFBcovid-2018.csv")) %>% data.table()
fb_2019 <- read.csv(paste0(data_folder,"postsFBcovid-2019.csv")) %>% data.table()
fb_2020 <- read.csv(paste0(data_folder,"postsFBcovid-2020.csv")) %>% data.table()

fb_2017$year <- 2017
fb_2018$year <- 2018
fb_2019$year <- 2019
fb_2020$year <- 2020

fb_posts <- rbind(fb_2017, fb_2018, fb_2019, fb_2020)
rm(fb_2017,fb_2018,fb_2019,fb_2020)

#On enlève les urls
fb_posts[,texteComplet := str_replace_all(texteComplet,"(?=http)[^ ]+", "")]
#On enlève les emojis
fb_posts[,texteComplet := str_replace_all(texteComplet, '[\U{1F300}-\U{1F6FF}]', "")]
#Le precedent code n'attrape pas certains emoji
fb_posts[,texteComplet := str_replace_all(texteComplet, '☀️', "")]
fb_posts[,texteComplet := str_replace_all(texteComplet, '🎉', "")] 

# Variable dichotomique pour le lien avec la science
fb_posts[grepl("(\\s)scien", texteComplet) | grepl("(\\s)chercheu", texteComplet),
         is_science := TRUE][is.na(is_science), is_science := FALSE]


# variable dichotomique pour la période COVID
fb_posts[year>=2020, since_COVID := TRUE][is.na(since_COVID), since_COVID := FALSE]

## Les quatre cas:
fb_posts[is_science == TRUE & since_COVID == TRUE, Status := "Science\npendant COVID"]
fb_posts[is_science == TRUE & since_COVID == FALSE, Status := "Science\navant COVID"]
fb_posts[is_science == FALSE & since_COVID == FALSE, Status := "Sans science\navant COVID"]
fb_posts[is_science == FALSE & since_COVID == TRUE, Status := "Sans science\npendant COVID"]
fb_posts[, Status2 := str_replace(Status, "\n", " ")]

# On s'assure que les dates sont bien restreintes.
fb_posts <- fb_posts[month(as.Date(jour)) %in% 1:6 & year(as.Date(jour)) %in% 2017:2020]


fb_posts[,semaine := week(as.POSIXct(dateHeure))]
fb_posts[ , day_of_year := as.numeric(strftime(jour, format = "%j"))]

fb_posts[,interaction := (likes + haha + colere + tristesse + wow + love + commentaires + partages),
         by = fbID]
fb_posts[, high_engagement := ifelse(log(interaction) > mean(log(interaction)), TRUE, FALSE), .(is_science,since_COVID)]

#### Sauvegarde d'un csv avec les champs sensibles retirés: #####
fb_posts[, list(media, dateHeure, postID, vignette, lien, type, partages, likes, love, wow, haha, colere, tristesse, 
                interaction, jour, year, semaine, day_of_year,  is_science, since_COVID, Status, Status2)] %>% 
  fwrite(file = paste0(accesssible_data_folder, "FB_data.csv"))

####  Sauvegarde avec tous les champs: #### 
save(fb_posts, file = paste0(data_folder, "dt_fb_posts_all_after_preprocessing.RData"))


#### Analyse morpho-syntaxique du sous-corpus "Science pendant Covid #### 
fb_sci_cov <-  fb_posts[is_science == TRUE & since_COVID == TRUE]

#Importation du modèle de langue 

model_dir <- paste0(getwd(),'/Udpipe_model')
if (!dir.exists(model_dir)) {dir.create(model_dir)}

# Télécharger du modèle de langue
udtarget <- udpipe_download_model(language = "french-gsd", model_dir = model_dir)

udmodel <- udpipe_load_model(file = udtarget$file_model)

#On crée la table avec de tokens annotés
dt_tagged_tokens_science <- 
  #Appliquer le partitionnement et l'analyse morphosyntaxique
  udpipe_annotate(
    udmodel,
    fb_sci_cov$texteComplet,
    doc_id = fb_sci_cov$postID) %>%
  as.data.table(.) %>%
  # on retire les tokens sans annotation
  .[!is.na(upos)]

#dt_tagged_tokens_science <- dt_tagged_tokens_science[,list(doc_id, token_id, token, lemma, upos)]

save(dt_tagged_tokens_science, file = paste0(data_folder, "dt_tagged_tokens_FB_science_in_COVID.RData"))

rm(dt_tagged_tokens_science)


